@@ -32,6 +32,8 @@
 #include "logout.h"
 #include "command.h"
 
+#define SESSION_STOCK_LOGOUT "session-logout"
+
 static gchar *halt_command[] =
 {
   HALT_COMMAND, NULL
@@ -159,16 +161,66 @@ refresh_screen (void)
   gdk_window_hide (window);
 }
 
+/* FIXME:
+ * 	Hackaround for Pango opening a separate display
+ * connection and doing a server grab while we have a grab
+ * on the primary display connection. This forces Pango
+ * to go ahead and do its font cache before we try to
+ * grab the server.
+ *
+ * c/f metacity/src/ui.c:meta_ui_init()
+ */
+static void
+force_pango_cache_init ()
+{
+	PangoFontMetrics     *metrics;
+	PangoLanguage        *lang;
+	PangoContext         *context;
+	PangoFontDescription *font_desc;
+
+	context = gdk_pango_context_get ();
+	lang = gtk_get_default_language ();
+	font_desc = pango_font_description_from_string ("Sans 12");
+	metrics = pango_context_get_metrics (context, font_desc, lang);
+
+	pango_font_metrics_unref (metrics);
+	pango_font_description_free (font_desc);
+}
+
+static inline void
+register_logout_stock_item (void)
+{
+	static gboolean registered = FALSE;
+
+	if (!registered) {
+		GtkIconFactory      *factory;
+		GtkIconSet          *quit_icons;
+
+		static GtkStockItem  logout_item [] = {
+			{ SESSION_STOCK_LOGOUT, N_("_Log Out"), 0, 0, GETTEXT_PACKAGE },
+		};
+
+		quit_icons = gtk_icon_factory_lookup_default (GTK_STOCK_QUIT);
+		factory = gtk_icon_factory_new ();
+		gtk_icon_factory_add (factory, SESSION_STOCK_LOGOUT, quit_icons);
+		gtk_icon_factory_add_default (factory);
+		gtk_stock_add_static (logout_item, 1);
+
+		registered = TRUE;
+	}
+}
+
 static gboolean
 display_gui (void)
 {
   GtkWidget *box;
   GtkWidget *toggle_button = NULL;
-  gint result;
+  gint response;
   gchar *s, *t;
   GtkWidget *halt = NULL;
   GtkWidget *reboot = NULL;
   GtkWidget *invisible;
+  gboolean   retval = FALSE;
 
   /* It's really bad here if someone else has the pointer
    * grabbed, so we first grab the pointer and keyboard
@@ -193,33 +245,36 @@ display_gui (void)
       sleep (1);
     }
 
+  force_pango_cache_init ();
+
   XGrabServer (GDK_DISPLAY ());
   iris ();
 
-  box = gnome_message_box_new (_("Really log out?"),
-			       GNOME_MESSAGE_BOX_QUESTION,
-			       GNOME_STOCK_BUTTON_HELP,
-			       GNOME_STOCK_BUTTON_NO,
-			       GNOME_STOCK_BUTTON_YES,
-			       NULL);
+  register_logout_stock_item ();
 
-  gtk_object_set (GTK_OBJECT (box),
-		  "type", GTK_WINDOW_POPUP,
-		  NULL);
+  box = gtk_message_dialog_new (NULL, 0,
+				GTK_MESSAGE_QUESTION,
+				GTK_BUTTONS_NONE,
+				_("Really log out?"));
 
-  gnome_dialog_set_default (GNOME_DIALOG (box), 2);
+  gtk_dialog_add_button (GTK_DIALOG (box), GTK_STOCK_HELP, GTK_RESPONSE_HELP);
+  gtk_dialog_add_button (GTK_DIALOG (box), GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
+  gtk_dialog_add_button (GTK_DIALOG (box), SESSION_STOCK_LOGOUT, GTK_RESPONSE_OK);
+
+  g_object_set (G_OBJECT (box), "type", GTK_WINDOW_POPUP, NULL);
+
+  gtk_dialog_set_default_response (GTK_DIALOG (box), GTK_RESPONSE_OK);
   gtk_window_set_position (GTK_WINDOW (box), GTK_WIN_POS_CENTER);
   gtk_window_set_policy (GTK_WINDOW (box), FALSE, FALSE, TRUE);
 
-  gnome_dialog_close_hides (GNOME_DIALOG (box), TRUE);
+  gtk_container_set_border_width (
+		GTK_CONTAINER (GTK_DIALOG (box)->vbox), GNOME_PAD);
 
-  gtk_container_set_border_width (GTK_CONTAINER (GNOME_DIALOG (box)->vbox),
-				  GNOME_PAD);
   if (!autosave)
     {
       toggle_button = gtk_check_button_new_with_label (_("Save current setup"));
       gtk_widget_show (toggle_button);
-      gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (box)->vbox),
+      gtk_box_pack_start (GTK_BOX (GTK_DIALOG (box)->vbox),
 			  toggle_button,
 			  FALSE, TRUE, 0);
     }
@@ -238,7 +293,7 @@ display_gui (void)
       GtkWidget *r;
 
       frame = gtk_frame_new (_("Action"));
-      gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (box)->vbox),
+      gtk_box_pack_start (GTK_BOX (GTK_DIALOG (box)->vbox),
 			  frame,
 			  FALSE, TRUE, GNOME_PAD_SMALL);
 
@@ -273,7 +328,7 @@ display_gui (void)
   if (GTK_WINDOW (box)->default_widget != NULL)
 	  gtk_widget_grab_focus (GTK_WINDOW (box)->default_widget);
 
-  result = gnome_dialog_run (GNOME_DIALOG (box));
+  response = gtk_dialog_run (GTK_DIALOG (box));
 
   refresh_screen ();
   XUngrabServer (GDK_DISPLAY ());
@@ -283,28 +338,35 @@ display_gui (void)
 
   gdk_flush ();
 
-  switch (result)
-    {
-    case 2:
-	/* We want to know if we should trash changes (and lose forever)
-	 * or save them */
-	if(!autosave)
-		save_selected = GTK_TOGGLE_BUTTON (toggle_button)->active;
-     if (halt)
+  switch (response) {
+    case GTK_RESPONSE_OK:
+      /* We want to know if we should trash changes (and lose forever)
+       * or save them */
+      if(!autosave)
+        save_selected = GTK_TOGGLE_BUTTON (toggle_button)->active;
+
+      if (halt)
 	{
 	  if (GTK_TOGGLE_BUTTON (halt)->active)
 	    action = HALT;
 	  else if (GTK_TOGGLE_BUTTON (reboot)->active)
 	    action = REBOOT;
 	}
-      return TRUE;
+      retval = TRUE;
+      break;
     default:
-    case 1:
-      return FALSE;
-    case 0:
+    case GTK_RESPONSE_CANCEL:
+      retval= FALSE;
+      break;
+    case GTK_RESPONSE_HELP:
       gnome_help_display("panelbasics.html#LOGGINGOUT", NULL, NULL);
-      return FALSE;
+      retval = FALSE;
+      break;
     }
+
+  gtk_widget_destroy (box);
+
+  return retval;
 }
 
 /* Display GUI if user wants it.  Returns TRUE if save should
